import {
  Asset,
  AnyChain,
  Ecosystem,
  Parachain,
  EvmParachain,
} from '@moonbeam-network/xcm-types';
import {
  BalanceConfigBuilder,
  ContractConfigBuilder,
  ExtrinsicConfigBuilder,
  AssetMinConfigBuilder,
  FeeConfigBuilder,
} from '@moonbeam-network/xcm-builder';

interface AssetConfigConstructorParams {
  asset: Asset;
  balance: BalanceConfigBuilder;
  contract?: ContractConfigBuilder;
  destination: AnyChain;
  destinationFee: DestinationFeeConfig;
  extrinsic?: ExtrinsicConfigBuilder;
  fee?: FeeAssetConfig;
  min?: AssetMinConfigBuilder;
}
interface DestinationFeeConfig {
  asset: Asset;
  amount: number | FeeConfigBuilder;
}
interface FeeAssetConfig {
  asset: Asset;
  balance: BalanceConfigBuilder;
}
declare class AssetConfig {
  readonly asset: Asset;
  readonly balance: BalanceConfigBuilder;
  readonly contract?: ContractConfigBuilder;
  readonly destination: AnyChain;
  readonly destinationFee: DestinationFeeConfig;
  readonly extrinsic?: ExtrinsicConfigBuilder;
  readonly fee?: FeeAssetConfig;
  readonly min?: AssetMinConfigBuilder;
  constructor({
    asset,
    balance,
    contract,
    destination,
    destinationFee,
    extrinsic,
    fee,
    min,
  }: AssetConfigConstructorParams);
}

interface TransferConfig {
  asset: Asset;
  source: ChainTransferConfig;
  destination: ChainTransferConfig;
}
interface ChainTransferConfig {
  chain: AnyChain;
  config: AssetConfig;
}

declare function ConfigBuilder(): {
  assets: (ecosystem?: Ecosystem) => {
    assets: Asset[];
    asset: (keyOrAsset: string | Asset) => {
      sourceChains: AnyChain[];
      source: (keyOrChain: string | AnyChain) => {
        destinationChains: AnyChain[];
        destination: (keyOrChain: string | AnyChain) => {
          build: () => TransferConfig;
        };
      };
    };
  };
};

declare const aca: Asset;
declare const astr: Asset;
declare const auq: Asset;
declare const ausd: Asset;
declare const bnc: Asset;
declare const cfg: Asset;
declare const crab: Asset;
declare const csm: Asset;
declare const dev: Asset;
declare const dot: Asset;
declare const eq: Asset;
declare const eqd: Asset;
declare const glmr: Asset;
declare const hdx: Asset;
declare const hko: Asset;
declare const ibtc: Asset;
declare const intr: Asset;
declare const kar: Asset;
declare const kbtc: Asset;
declare const kint: Asset;
declare const kma: Asset;
declare const ksm: Asset;
declare const lit: Asset;
declare const mgx: Asset;
declare const movr: Asset;
declare const nodl: Asset;
declare const para: Asset;
declare const paring: Asset;
declare const pha: Asset;
declare const ring: Asset;
declare const rmrk: Asset;
declare const sdn: Asset;
declare const soon: Asset;
declare const teer: Asset;
declare const tt1: Asset;
declare const tur: Asset;
declare const unit: Asset;
declare const usdt: Asset;
declare const xrt: Asset;
declare const assetsList: Asset[];
declare const assetsMap: Map<string, Asset>;

declare const acala: Parachain;
declare const alphanetAssetHub: Parachain;
declare const alphanetRelay: Parachain;
declare const astar: Parachain;
declare const bifrostKusama: Parachain;
declare const bifrostPolkadot: Parachain;
declare const calamari: Parachain;
declare const centrifuge: Parachain;
declare const crustShadow: Parachain;
declare const darwinia: EvmParachain;
declare const darwiniaCrab: EvmParachain;
declare const darwiniaPangoro: EvmParachain;
declare const equilibrium: Parachain;
declare const equilibriumAlphanet: Parachain;
declare const hydraDX: Parachain;
declare const interlay: Parachain;
declare const integritee: Parachain;
declare const karura: Parachain;
declare const khala: Parachain;
declare const kintsugi: Parachain;
declare const kusama: Parachain;
declare const kusamaAssetHub: Parachain;
declare const litentryAlphanet: Parachain;
declare const litmus: Parachain;
declare const mangataKusama: Parachain;
declare const moonbaseAlpha: EvmParachain;
declare const moonbaseBeta: EvmParachain;
declare const moonbeam: EvmParachain;
declare const moonriver: EvmParachain;
declare const nodleEden: Parachain;
declare const parallel: Parachain;
declare const parallelHeiko: Parachain;
declare const phala: Parachain;
declare const polkadot: Parachain;
declare const polkadotAssetHub: Parachain;
declare const robonomics: Parachain;
declare const shiden: Parachain;
declare const turing: Parachain;
declare const turingAlphanet: Parachain;
declare const subsocialAlphanet: Parachain;
declare const uniqueAlpha: Parachain;
declare const chainsList: AnyChain[];
declare const chainsMap: Map<string, AnyChain>;

interface ChainConfigConstructorParams {
  assets: AssetConfig[];
  chain: AnyChain;
}
declare class ChainConfig {
  #private;
  readonly chain: AnyChain;
  constructor({ assets, chain }: ChainConfigConstructorParams);
  getAssetsConfigs(): AssetConfig[];
  getAssetConfigs(asset: Asset): AssetConfig[];
  getAssetDestinations(asset: Asset): AnyChain[];
  getAssetDestinationConfig(asset: Asset, destination: AnyChain): AssetConfig;
}

export {
  AssetConfig,
  AssetConfigConstructorParams,
  ChainConfig,
  ChainConfigConstructorParams,
  ChainTransferConfig,
  ConfigBuilder,
  DestinationFeeConfig,
  FeeAssetConfig,
  TransferConfig,
  aca,
  acala,
  alphanetAssetHub,
  alphanetRelay,
  assetsList,
  assetsMap,
  astar,
  astr,
  auq,
  ausd,
  bifrostKusama,
  bifrostPolkadot,
  bnc,
  calamari,
  centrifuge,
  cfg,
  chainsList,
  chainsMap,
  crab,
  crustShadow,
  csm,
  darwinia,
  darwiniaCrab,
  darwiniaPangoro,
  dev,
  dot,
  eq,
  eqd,
  equilibrium,
  equilibriumAlphanet,
  glmr,
  hdx,
  hko,
  hydraDX,
  ibtc,
  integritee,
  interlay,
  intr,
  kar,
  karura,
  kbtc,
  khala,
  kint,
  kintsugi,
  kma,
  ksm,
  kusama,
  kusamaAssetHub,
  lit,
  litentryAlphanet,
  litmus,
  mangataKusama,
  mgx,
  moonbaseAlpha,
  moonbaseBeta,
  moonbeam,
  moonriver,
  movr,
  nodl,
  nodleEden,
  para,
  parallel,
  parallelHeiko,
  paring,
  pha,
  phala,
  polkadot,
  polkadotAssetHub,
  ring,
  rmrk,
  robonomics,
  sdn,
  shiden,
  soon,
  subsocialAlphanet,
  teer,
  tt1,
  tur,
  turing,
  turingAlphanet,
  uniqueAlpha,
  unit,
  usdt,
  xrt,
};
